/*
    Dump the image header info of any supported file format
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <ifl/iflTypeNames.h>
#include <ifl/iflCdefs.h>


void 
printInfo(iflFile* file, int indexSpecified, int verbose)
{
    iflSize dims;
    iflFormat* format;
    int nimg;
    int firstimg, lastimg, defaultimg, imgi;
    
    /* print image name */
    const char* name = iflFileGetFileName(file);
    if (verbose)
        printf("Image file:\t\t%s\n", name);

    /* get image dimensions */
    iflFileGetDimensions(file, &dims);

    /* print file format */
    format = iflFileGetFormat(file);
    if (verbose)
	printf("File format:\t\t%s\n", iflFormatGetDescription(format));
    else 
	printf("%s:\t%s format (%s %dx%d)\n", name, iflFormatGetName(format), 
	       iflColorModelName(iflFileGetColorModel(file)), dims.x, dims.y);
    
    /* the rest is only for long format */
    if (!verbose) return;
	
    nimg = iflFileGetNumImgs(file);
    if (nimg > 1) printf("Number of sub-images:\t%d\n", nimg);

    /* will print all images, unless an index was specified */
    firstimg = 0, lastimg = nimg-1, defaultimg = -1;
    if (indexSpecified)
	firstimg = lastimg = iflFileGetCurrentImg(file);
    else
	defaultimg = iflFileGetCurrentImg(file);
    
    for (imgi = firstimg; imgi <= lastimg; imgi++) {
	iflSize dims;
	iflSize pgDims;
	double min, max;
	iflCompression compress;

	iflFileSetCurrentImg(file, imgi);	
	
	/* print sub-image index (if more than 1) */
	if (nimg > 1)
	    printf("\nSub-image:\t\t%d%s\n", imgi,
		   imgi == defaultimg ? "\t\t(default)" : "");

	/* print image dimensions */
	iflFileGetDimensions(file, &dims);
	printf("Dimensions (w,h,z,c):\t"
	       "%d, %d, %d, %d\n", dims.x, dims.y, dims.z, dims.c);

	/* print image pagesize */
	iflFileGetPageDimensions(file, &pgDims);
	printf("Page Size (w,h,z,c):\t"
	       "%d, %d, %d, %d\n", pgDims.x, pgDims.y, pgDims.z, pgDims.c);

	/* print the rest of the image info */
	printf("Data type:\t\t%s\n", iflDataTypeName(iflFileGetDataType(file)));
	printf("Dimension order:\t%s\n", iflOrderName(iflFileGetOrder(file)));
	printf("Color model:\t\t%s\n", 
	       iflColorModelName(iflFileGetColorModel(file)));
	printf("Orientation:\t\t%s\n", 
	       iflOrientationName(iflFileGetOrientation(file)));

	/* print min and max values (if present) */
	if (iflFileGetStatMinMax(file, &min, &max) == iflOKAY) 
	    printf("Statistical min/max:\t%g - %g\n", min, max);
	if (iflFileGetScaleMinMax(file, &min, &max) == iflOKAY) 
	    printf("Display min/max:\t%g - %g\n", min, max);

	/* print compression (if any) */
	compress = iflFileGetCompression(file);
	if (compress != iflNoCompression) 
	    printf("Data Compression:\t%s\n", iflCompressionName(compress));
    }
}


void
main(int argc, char* argv[])
{
    /* process command line arguments */

    int numFiles, idx;
    int usage = FALSE, verbose = FALSE, c;
    char** names;
    
    while ((c = getopt(argc, argv, "v")) != -1) {
        switch (c) {
        case 'v':
            verbose = TRUE;
            break;
        case '?':
            usage = TRUE;
            break;
	}
    }
	
    numFiles = argc - optind;
    names = argv+optind;

    if (usage || numFiles < 1) {
        fprintf(stderr, "%s [-v] <image-file-1> [...]\n", argv[0]);
        exit(0);
    }

    /* fore each image file, try to open the file and print info on it */

    for (idx = 0; idx < numFiles; idx++) {

	iflStatus sts;
	iflFile* file = iflFileOpen(names[idx], O_RDONLY, &sts);
	if (file == NULL) {
	    char msg[1024];
	    iflStatusToString(sts, msg, sizeof msg);
	    fprintf(stderr, "Unable to open \"%s\": %s\n", names[idx], msg);
 	}
	else {
	    /* leave a gap between files */		
	    if (idx > 0 && verbose) printf("\n\n");
    
	    printInfo(file, strchr(names[idx], ':') != NULL, verbose ||
							     numFiles == 1);
    
	    if (file != NULL) iflFileClose(file, 0);
	}
    }
}
