// this program generates a variety of luminance test pattern images
// 
// Originally written by Paul Haeberli, it has been extensively modified
// and in no way should its current form reflect on Paul's capabilities.

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <getopt.h>
#include <ifl/iflFile.h>

#if defined(_WIN32)
#define M_PI 3.141592654f
#define strncasecmp strnicmp
#define random() rand()
#endif


inline 
float degToRad(float d) 
{ 
    return d*M_PI/180.0f; 
}


inline float 
flerp(float f0, float f1, float p)
{
    return ((f0*(1.0f-p))+(f1*p));
}


float 
quadratic(float x)	
{
    // 3rd order (quadratic) b-spline 
    float t;

    if (x<-1.5f) return 0.f;
    if (x<-0.5f) {
	t = x+1.5f; 
	return 0.5f*t*t;
    }
    if (x<0.5f) return 0.75f-x*x;
    if (x<1.5f) {
	t = x-1.5f; 
	return 0.5f*t*t;
    }
    return 0.0f;
}


inline float 
gauss(float f)
{
    float val; 
    val = quadratic(3.f*(f-.5f)) / .75f;
    return val;
}


float 
rolffunc(float x)
{
    float y;

    if (x < -1.5f)  
	y = 0.0f;
    else if ((x >= -1.5f) && (x < -.5f))  
	y = 0.5f * (x + 1.5f) * (x + 1.5f);
    else if ((x >= -0.5f) && (x < 0.5f))  
	y = 0.5f + (x + 0.5f) - (x + 0.5f) * (x + 0.5f);
    else if ((x >= 0.5f) && (x < 1.5f))  
	y = 0.5f - (x - 0.5f) + 0.5f * (x - 0.5f) * (x - 0.5f);
    else  
	y = 0.0f;
    return y;
}


inline float 
rolf(float x)
{
    return (4.f/3.f) * rolffunc(x*3.f - 1.5f);
}


inline double 
logbase(double n, double b)
{
   return log10(n) / log10(b);
}


inline void
putpix(unsigned char*& ptr, int val) 
{ 
    *ptr++ = val; 
}


inline void
putrow(unsigned char*& ptr, unsigned char* buf, int count) 
{ 
    for (int i = 0; i < count; i++) putpix(ptr, buf[i]); 
}


void
main(int argc, char** argv)
{
    enum {
	XRAMP, YRAMP, CONE, WHITE, STRIPES, DOTS, CHECKS, GRID, GAMMATEST, 
	GAUSSX, GAUSSY, CONTRAST, BLACK, RANDOM, CIRCLE, ROLF, LINETEST, BAYER, 
	DIRCONE, PRINTTEST, EXPRAMP, SHARPTHRESH, LIGHT, KERN242
    };
    
    struct Pattern { 
	char* name;
	int code;
    } patterns[] = {
	{ "bayer", BAYER }, 
	{ "black", BLACK }, 
	{ "checks", CHECKS }, 
	{ "circle", CIRCLE }, 
	{ "cone", CONE }, 
	{ "contrast", CONTRAST }, 
	{ "dircone", DIRCONE }, 
	{ "dots", DOTS }, 
	{ "expramp", EXPRAMP }, 
	{ "gammatest", GAMMATEST }, 
	{ "gaussx", GAUSSX }, 
	{ "gaussy", GAUSSY }, 
	{ "grid", GRID }, 
	{ "kern242", KERN242 }, 
	{ "light", LIGHT }, 
	{ "linetest", LINETEST }, 
	{ "printtest", PRINTTEST }, 
	{ "random", RANDOM }, 
	{ "rolf", ROLF }, 
	{ "sharpthresh", SHARPTHRESH }, 
	{ "stripes", STRIPES }, 
	{ "white", WHITE }, 
	{ "xramp", XRAMP }, 
	{ "yramp", YRAMP }, 
	{ NULL, 0 }
    };

    // process arguments
    int usage=FALSE, xsize=256, ysize=256, pattern=XRAMP;
    int c;
    while ((c = getopt(argc, argv, "s:p:")) != -1) {
        switch (c) {
	case 's':
	    sscanf(optarg, "%d,%d", &xsize, &ysize);
	    break;
        case 'p': 
	  {
	    int len = strlen(optarg);
	    for (Pattern* pat = patterns; pat->name != NULL; pat++) {
		if (strncasecmp(optarg, pat->name, len) == 0)
            	    { pattern = pat->code; break; }
	    }
	    if (pat->name == NULL) {
		usage = 1;
		fprintf(stderr, "Unknown pattern type: %s; use one of:.\n", 
			optarg);
		for (Pattern* pat = patterns; pat->name != NULL; pat++) 
		    fprintf(stderr, "    %s\n", pat->name);
	    }
	    break;
	  }
	default:
	    usage = TRUE;
	    break;
	}
    }

    if (usage || argc-optind != 1) {
	fprintf(stderr, "usage: %s [-s<size x,y> -p<pattern>] outimage\n", 
		argv[0]);
	exit(1);
    }

    // this pattern expects a specific size
    if (pattern == PRINTTEST) {
	xsize = 32;
        ysize = 2;
    } 

    // create the file
    iflStatus err;
    iflSize size(xsize, ysize, 1);
    iflFileConfig fc(&size, iflUChar);
    iflFile* out = iflFile::create(argv[optind], NULL, &fc, NULL, &err);
    if (out == NULL) {
        char errmsg[1024];
        iflStatusToString(err, errmsg, sizeof errmsg);
        fprintf(stderr, "unable to open \"%s\": %s\n", argv[optind], errmsg);
	exit(1);
    }

    // create output buffer
    unsigned char* obuff = new unsigned char [xsize*ysize];
    unsigned char* optr = obuff;

    // some line buffers
    unsigned char buf[8192];
    unsigned char wbuf[8192];
    unsigned char bbuf[8192];

    // image dimension counters
    int x, y;

    // fill output buffer with pattern
    switch (pattern) {

    case XRAMP:
	for (x=0; x<xsize; x++) 
	    buf[x] = (255*x)/(xsize-1);
	for (y=0; y<ysize; y++)
	    putrow(optr, buf, xsize);
	break;

    case YRAMP:
	for (y=0; y<ysize; y++)
	    for (x=0; x<xsize; x++) 
		putpix(optr, (255*y)/(ysize-1));
	break;

    case CONE:
	for (y=0; y<ysize; y++)
	    for (x=0; x<xsize; x++) {
		float dx = (float)(xsize/2)-x;
		float dy = (float)(ysize/2)-y;
		float d = sqrtf(dx*dx+dy*dy);
		putpix(optr, (int)(512*d/(1.5f*xsize)));
	    }
	break;

    case STRIPES:
	for (x=0; x<xsize; x++) 
	    wbuf[x] = 255;
	for (x=0; x<xsize; x++) 
	    bbuf[x] = 0;
	for (y=0; y<ysize; y++) {
	    if ((y%2) < 1)
		putrow(optr, bbuf, xsize);
	    else
		putrow(optr, wbuf, xsize);
	}
	break;

    case DOTS:
	for (y=0; y<ysize; y++) {
	    for (x=0; x<xsize; x++) {
		if ((random() % 101) == 35) 
		    putpix(optr, 128+(random()%128));
		else
		    putpix(optr, 0);
	    }
	}
	break;

    case CHECKS:
	for (y=0; y<ysize; y++) {
	    for (x=0; x<xsize; x++) {
		if ((x+y) % 2)
		    putpix(optr, 255);
		else
		    putpix(optr, 0);
	    }
	}
	break;

    case GRID:
	for (y=0; y<ysize; y++) {
	    if (y%10 == 0) {
		for (x=0; x<xsize; x++) 
		    putpix(optr, 0);
	    } else {
		for (x=0; x<xsize; x++) {
		    if ((x%10) == 0)
			putpix(optr, 0);
		    else
			putpix(optr, 255);
		}
	    }
	}
	break;

    case GAMMATEST:
	for (y=0; y<ysize; y++) {
	    if ( (y>>3)&1 ) {
		for (x=0; x<xsize; x++) 
		    putpix(optr, 127);
	    } else {
		for (x=0; x<xsize; x++) 
		    putpix(optr, 254*(((x/2)+(y/2))&1));
	    }
	}
	break;

    case GAUSSX:
	for (x=0; x<xsize; x++) 
	    buf[x] = (unsigned char)(255*gauss(x/(xsize-1.0f)));
	for (y=0; y<ysize; y++) 
	    putrow(optr, buf, xsize);
	break;

    case GAUSSY:
	for (y=0; y<ysize; y++) {
	    int val = (int)(255*gauss(y/(ysize-1.0f)));
	    for (x=0; x<xsize; x++) 
		buf[x] = val;
	    putrow(optr, buf, xsize);
	}
	break;

    case CONTRAST:
	for (y=0; y<ysize; y++) {
	    for (x=0; x<xsize; x++) {
		if ((x%2) == 0) 
		    putpix(optr, 64);
		else
		    putpix(optr, 64+((y*(255-64))/(ysize-1)));
	    }
	}
	break;

    case BLACK:
	for (x=0; x<xsize; x++) 
	    buf[x] = 0;
	for (y=0; y<ysize; y++) 
	    putrow(optr, buf, xsize);
	break;

    case WHITE:
	for (x=0; x<xsize; x++) 
	    buf[x] = 255;
	for (y=0; y<ysize; y++) 
	    putrow(optr, buf, xsize);
	break;

    case RANDOM:
	for (y=0; y<ysize; y++) 
	    for (x=0; x<xsize; x++) 
		putpix(optr, random()&0xff);
	break;

    case CIRCLE:
	for (y=0; y<ysize; y++)
	    for (x=0; x<xsize; x++) {
		float dx = ((x+0.5f)/xsize)-0.5f;
		float dy = ((y+0.5f)/ysize)-0.5f;
		float d = sqrtf(dx*dx+dy*dy);
		if (d>0.5f)
		    putpix(optr, 0);
		else
		    putpix(optr, 255);
	    }
	break;

    case ROLF:
	for (x=0; x<xsize; x++) 
	    buf[x] = (unsigned char)(255*rolf(x/(xsize-1.0f)));
	for (y=0; y<ysize; y++)
	    putrow(optr, buf, xsize);
	break;

    case LINETEST:
	for (y=0; y<ysize; y++) {
	    float fy = y/(ysize-1.0f);
	    for (x=0; x<xsize; x++) {
		int val = int(127*(1.0f+sin(degToRad(60.0f*(1.0f+fy*2.0f)*x))));
		putpix(optr, val);
	    }
	}
	break;

    case BAYER:
      { short dithmat[4][4] = {
	    0, 8, 2, 10,
	    12, 4, 14, 6,
	    3, 11, 1, 9,
	    15, 7, 13, 5,
	};
	for (y=0; y<ysize; y++) {
	    for (x=0; x<xsize; x++) 
		putpix(optr, 16*dithmat[x&3][y&3]);
	}
	break;
      }

    case DIRCONE:
	for (y=0; y<ysize; y++) {
	    for (x=0; x<xsize; x++) {
		float dx = ((x+0.5f)/xsize)-0.5f;
		float dy = ((y+0.5f)/ysize)-0.5f;
		int val = int(255*atan2(dx,dy)/(2.0f*3.1415926535f));
		val = 255-val;
		val += 64;
		if (val>255)
		    val -= 256;
		putpix(optr, val);
	    }
	}
	break;

    case PRINTTEST:
	for (x=0; x<31; x++) {
	    putpix(optr, 255-2*x);
	}
	putpix(optr, 128);
	for (x=0; x<31; x++) {
	    putpix(optr, 2*x);
	}
	putpix(optr, 128);
	break;

    case EXPRAMP:
      { float max = (float)logbase(255.0f, 2.0f);
	for (x=0; x<xsize; x++) {
	    float f = max*flerp(0.2f, 1.0f,x/(xsize-1.0f));
	    f = (float)pow(2.0f,f);
	    buf[x] = (unsigned char)(f+0.5f);
	}
	for (y=0; y<ysize; y++) 
	    putrow(optr, buf, xsize);
	break;
      }

    case SHARPTHRESH:
      { short sharpthresh[8][8] = {
	    57, 63, 41, 49, 55, 61, 44, 51,
	    35, 55, 21, 29, 33, 53, 24, 31,
	    27, 47, 9,  13, 25, 45, 11, 15,
	    19, 39, 1,  5,  17, 37, 3,  7,
	    40, 48, 54, 60, 42, 50, 56, 62,
	    20, 28, 32, 52, 22, 30, 34, 54,
	    8,  12, 24, 44, 10, 14, 26, 46,
	    0,  4,  16, 36, 2,  6,  18, 38,
	};
	for (y=0; y<ysize; y++) {
	    for (x=0; x<xsize; x++) 
		putpix(optr, 4*sharpthresh[x&7][y&7]);
	}
	break;
      }

    case LIGHT:
	for (y=0; y<ysize; y++) {
	    float fy = 2.0f*((y/(ysize-1.0f))-0.5f);
	    fy = fy*fy;
	    float d = 1.0f;
	    d = d*d;
	    for (x=0; x<xsize; x++) {
		float fx = 2.0f*((x/(xsize-1.0f))-0.5f);
		fx = fx*fx;
		int val = int((d*255)/(fx+fy+d));
		putpix(optr, val);
	    }
	}
	break;

    case KERN242:
      {	short kern242[3][3] = {
	    64, 128, 64,
	    128, 255, 128,
	    64, 128, 64,
	};
	for (y=0; y<ysize; y++) {
	    for (x=0; x<xsize; x++) {
		putpix(optr, kern242[y%3][x%3]);
	    }
	}
	break;
      }
    }

    // write data to file
    iflConfig cfg(iflUChar, iflInterleaved);
    err = out->setTile(0, 0, 0, xsize, ysize, 1, obuff, &cfg);
    if (err != iflOKAY) {
        char errmsg[1024];
        iflStatusToString(err, errmsg, sizeof errmsg);
        fprintf(stderr, "Failed to write \"%s\": %s\n", argv[optind], errmsg);
	exit(1);
    }

    out->close();

    exit(0);
}
