//=======================================================================
// Arcball class implementation.
// 19/7/99, Diego Nehab
//=======================================================================

#ifndef ARCBALL_H
#define ARCBALL_H

#include "quaternion.h"

//=======================================================================
// Arcball Class
//=======================================================================
class Arcball {
public:
    //---------------------------------------------------------------------
    // Constructor
    // Input
    //     xc, yc, r: ball center and radius. defaults to origin, unitary.
    //     qt: initial rotation. defaults to identity.
    //---------------------------------------------------------------------
    Arcball(double xc = 0, double yc = 0, double r = 1, 
            const Quaternion &qt = Quaternion(0, 0, 0, 1));

    //---------------------------------------------------------------------
    // Adjust ball to new dimensions
    // Input
    //     xc, yc, r: ball center and radius
    //---------------------------------------------------------------------
    void Reset(double xc, double yc, double r);

    //---------------------------------------------------------------------
    // Mouse down event
    // Input
    //     x, y: coordinates
    //---------------------------------------------------------------------
    void Down(double x, double y);

    //---------------------------------------------------------------------
    // Mouse motion event
    // Input
    //     x, y: coordinates
    //---------------------------------------------------------------------
    void Motion(double x, double y);

    //---------------------------------------------------------------------
    // Mouse down event
    // Input
    //     x, y: coordinates
    //---------------------------------------------------------------------
    void Up(double x, double y);

    //---------------------------------------------------------------------
    // Set rotation to arbitrary value
    // Input
    //     qt: quaternion for desired rotation
    //---------------------------------------------------------------------
    void Set(const Quaternion &qt = Quaternion(0, 0, 0, 1));

    //---------------------------------------------------------------------
    // Get cumulative rotation
    //---------------------------------------------------------------------
    double Rotation(Vector3 *v) const;
    double Rotation(double *x, double *y, double *z) const;

private:

    Vector3 getv(double x, double y);

    // are we dragging?
    int drag;
    // ball center and radius
    double xc, yc, r;
    // initial and final points
    Vector3 ei, ef;
    // current and cumulative rotations
    Quaternion qt, q;
};

//=======================================================================
// Public Methods
//=======================================================================
inline 
Arcball::Arcball(double uxc, double uyc, double ur, const Quaternion &uqt)
{
    drag = false;
    Reset(uxc, uyc, ur);
    Set(uqt);
}

inline void
Arcball::Reset(double uxc, double uyc, double ur)
{
    xc = uxc;
    yc = uyc;
    r = ur;
    q = Quaternion(0, 0, 0, 1);
}

inline void
Arcball::Set(const Quaternion &uqt)
{
    qt = uqt;
}

inline void 
Arcball::Down(double x, double y)
{
    // get mouse down position in ball
    ei = getv(x, y);
    // reset current rotation
    q = Quaternion(0, 0, 0, 1);
    // begin dragging
    drag = 1;
}

inline void
Arcball::Up(double x, double y)
{
    // make sure we move to final position
    Motion(x, y);
    // add current rotation into cumulative rotation
    qt = q * qt;
    // finish dragging
    drag = 0;
    // reset current rotation
    q = Quaternion(0, 0, 0, 1);
}

inline void
Arcball::Motion(double x, double y)
{
    // if no dragging, ignore
    if (!drag) return;
    // get mouse position in ball
    ef = getv(x, y);
    // get rotation axis
    Vector3 v = Cross(ei, ef);
    // get rotation angle
    double a = Dot(ei, ef);
    // create rotation from mouse down position to current position
    q = Quaternion(v[0], v[1], v[2], a);
}

inline double
Arcball::Rotation(Vector3 *v) const
{
    // add current rotation to cumulative rotation
    Quaternion qr = q * qt;
    // return total rotation
    return qr.Rotation(v);
}

inline double
Arcball::Rotation(double *x, double *y, double *z) const
{
    // add current rotation to cumulative rotation
    Quaternion qr = q * qt;
    // return total rotation
    return qr.Rotation(x, y, z);
}

//=======================================================================
// Private Methods
//=======================================================================
inline Vector3
Arcball::getv(double x, double y)
{
    x = (x - xc) / r;
    y = (y - yc) / r;
    double n = x*x + y*y;
    // if point is out of sphere area
    if (n >= 1) {
        n = sqrt(n);
        return Vector3(x/n, y/n, 0);
    // if point is in sphere area
    } else return Vector3(x, y, sqrt(1-n));
}

#endif // ARCBALL_H
