#ifndef IMAGEWINDOW_H
#define IMAGEWINDOW_H

#include "window.h"
#include "buffer.h"
#include "scalar.h"
#include "glutwm.h"
#include "error.h"

template <class T> 
class ImageWindow: public Window {
    public:
        ImageWindow();
        virtual ~ImageWindow();

        int Store(Buffer<T> &other);
        int ScreenToImage(double *x, double *y);
        int ImageToScreen(double *x, double *y);
        Buffer<T> *GetBuffer(void);
        const Buffer<T> *GetBuffer(void) const;
        void SetBuffer(Buffer<T> *buffer);
        int Fit(void);

        virtual void Display(void);
        virtual void Button(int button, int state, int x, int y); 
        virtual void Motion(int x, int y); 
        virtual void Keyboard(unsigned char c, int x, int y); 

        bool IsDragging(void);
    private:
        Buffer<T> *buffer;
        double cx, cy;         // center of displayed part of image
        int z;                 // zoom factor relative to perfect fit
        bool drag;
        double drag_cx, drag_cy;
        double drag_x, drag_y;
        double drag_zx, drag_zy;
        void DrawBuffer(void);
        void DrawBackground(void);
        void Zoom(void);
        void GetDrawParameters(int *ix, int *iy, int *iw, int *ih, 
                double *px, double *py, double *zx, double *zy);

};

typedef ImageWindow<Uchar> ImageWindowUchar;
typedef ImageWindow<Ushort> ImageWindowUshort;
typedef ImageWindow<Ulong> ImageWindowUlong;
typedef ImageWindow<float> ImageWindowFloat;
typedef ImageWindow<double> ImageWindowDouble;

template <class T>
ImageWindow<T>::ImageWindow()
{
    drag = false;
    buffer = NULL;
}

template <class T>
Buffer<T>*
ImageWindow<T>::
GetBuffer(void)
{
    return buffer;
}

template <class T>
const Buffer<T>*
ImageWindow<T>::
GetBuffer(void) const
{
    return buffer;
}

template <class T>
void 
ImageWindow<T>::
SetBuffer(Buffer<T> *_buffer) 
{
    buffer = _buffer;
}

template <class T>
bool
ImageWindow<T>::
IsDragging(void)
{
    return drag;
}

template <class T>
int 
ImageWindow<T>::
Store(Buffer<T> &other)
{
    int old = glutGetWindow();
    glutSetWindow(GetId());
    int width = GetWidth();
    int height = GetHeight();
    if (!other.Resize(width, height, 3)) return 0;
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    GlutWMReadPixels(0, 0, width, height, GL_RGB, other.GetBuffer());
    if (old > 0) glutSetWindow(old);
    return 1;
}

template <class T>
int 
ImageWindow<T>::
Fit(void)
{
    if (!buffer) {
        errorf(("No buffer setup"));
        return 0;
    }
    z = 1;
    cx = buffer->GetWidth()/2;
    cy = buffer->GetHeight()/2;
    return 1;
}

template <class T>
ImageWindow<T>::
~ImageWindow()
{
}

template <class T>
int 
ImageWindow<T>::
ScreenToImage(double *x, double *y)
{
    if (!buffer) {
        errorf(("No buffer setup"));
        return 0;
    }
    int ix, iy, iw, ih;
    double px, py, zx, zy;
    GetDrawParameters(&ix, &iy, &iw, &ih, &px, &py, &zx, &zy);
    *x -= px;
    *y -= py;
    *x /= zx;
    *y /= zy;
    *x += ix;
    *y += iy;
    return 1;
}

template <class T>
int 
ImageWindow<T>::
ImageToScreen(double *x, double *y)
{
    if (!buffer) {
        errorf(("No buffer setup"));
        return 0;
    }
    int ix, iy, iw, ih;
    double px, py, zx, zy;
    GetDrawParameters(&ix, &iy, &iw, &ih, &px, &py, &zx, &zy);
    *x -= ix;
    *y -= iy;
    *x *= zx;
    *y *= zy;
    *x += px;
    *y += py;
    return 1;
}

template <class T>
void 
ImageWindow<T>::
Display(void)
{
    int w = GetWidth(), h = GetHeight();
    int x0 = GetX0(), y0 = GetY0();
    // you never know who changed your stuff
    glViewport(x0, y0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, w, 0, h, -1, 1);
    // again, you never know 
    glShadeModel(GL_FLAT);
    // draw the image
    DrawBackground();
    DrawBuffer();
}

template <class T>
void 
ImageWindow<T>::
Keyboard(unsigned char c, int x, int y)
{
    if (c == 'f') { 
        Fit();
        GlutWMPostRedisplay();
    } else Window::Keyboard(c, x, y);
}

#define SIDE 15
template <class T>
void
ImageWindow<T>::
DrawBackground(void)
{
    glClearColor(1, 1, 1, 1);
    glClear(GL_COLOR_BUFFER_BIT);
    glColor3f(0.5, 0.5, 0.5);
    glBegin(GL_QUADS);
    for (int i = 0; i*SIDE < GetWidth(); i++) {
        for (int j = 0; j*SIDE < GetHeight(); j++) {
            if ((i + j) & 1) {
                glVertex2i(i*SIDE, j*SIDE);
                glVertex2i(i*SIDE, (j+1)*SIDE);
                glVertex2i((i+1)*SIDE, (j+1)*SIDE);
                glVertex2i((i+1)*SIDE, j*SIDE);
            }
        }
    }
    glEnd();
}

template <class T>
void 
ImageWindow<T>::
Button(int button, int state, int x, int y) 
{ 
    if (!buffer) return;
    if (button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN) {
        if (z < 33) {
            // find out where image point under mouse pointer
            // will show up after zoom
            double nx = x, ny = y;
            ScreenToImage(&nx, &ny);
            z++;
            ImageToScreen(&nx, &ny);
            // translate center of image to make sure it remains
            // in the same screen position after zoom
            int ix, iy, iw, ih;
            double px, py, zx, zy;
            GetDrawParameters(&ix, &iy, &iw, &ih, &px, &py, &zx, &zy);
            cx = cx + (nx - (double) x)/zx;
            cy = cy + (ny - (double) y)/zy;
            GlutWMPostRedisplay();
        }
    }
    if (button == GLUT_MIDDLE_BUTTON && state == GLUT_DOWN) {
        if (z > 1) {
            // find out where image point under mouse pointer
            // will show up after zoom
            double nx = x, ny = y;
            ScreenToImage(&nx, &ny);
            z--;
            ImageToScreen(&nx, &ny);
            // translate center of image to make sure it remains
            // in the same screen position after zoom
            int ix, iy, iw, ih;
            double px, py, zx, zy;
            GetDrawParameters(&ix, &iy, &iw, &ih, &px, &py, &zx, &zy);
            cx = cx + (nx - (double) x)/zx;
            cy = cy + (ny - (double) y)/zy;
            GlutWMPostRedisplay();
        }
    }
    if (button == GLUT_LEFT_BUTTON) {
        if (state == GLUT_DOWN) {
            int ix, iy, iw, ih;
            double px, py, zx, zy;
            GetDrawParameters(&ix, &iy, &iw, &ih, &px, &py, &zx, &zy);
            drag_cx = cx;
            drag_cy = cy;
            drag_zx = zx;
            drag_zy = zy;
            drag_x = x;
            drag_y = y;
            drag = true;
        } else {
            drag = false;
            cx = drag_cx - ((double) x - drag_x)/drag_zx;
            cy = drag_cy - ((double) y - drag_y)/drag_zy;
            GlutWMPostRedisplay();
        }
    }
}

template <class T>
void 
ImageWindow<T>::
Motion(int x, int y)
{
    if (!buffer) return;
    if (drag) {
        cx = drag_cx - ((double) x - drag_x)/drag_zx;
        cy = drag_cy - ((double) y - drag_y)/drag_zy;
        GlutWMPostRedisplay();
    }
}


template <class T>
void
ImageWindow<T>::
GetDrawParameters(int *ix, int *iy, int *iw, int *ih, 
        double *px, double *py, double *zx, double *zy)
{
    if (!buffer) return;
    int bw = buffer->GetWidth(), bh = buffer->GetHeight();
    int sw = GetWidth(), sh = GetHeight();
    // get best fit;
    double a = (double)(sw)/bw, b = (double)(sh)/bh;
    *zx = *zy = a < b ? a : b;
    *px = (sw - bw*(*zx))/2.0; *py = (sh - bh*(*zy))/2.0;
    // scale best fit to account for zoom (keeping center)
    *zx *= z; *zy *= z;
    double dx = *px - sw/2.0; double dy = *py - sh/2.0;
    dx *= z; dy *= z;
    *px = dx + sw/2.0; *py = dy + sh/2.0;
    // translate scaled best fit to new center to account for pan
    *px += (bw/2.0 - cx)*(*zx); *py += (bh/2.0 - cy)*(*zy);
    // clip image area to screen
    *ix = (int)(-(*px)/(*zx));
    *ix = *ix > 0 ? *ix : 0;
    *iy = (int)(-(*py)/(*zy));
    *iy = *iy > 0 ? *iy : 0;
    *iw = (int)(sw/(*zx)+1); 
    *iw = *iw < bw - *ix ? *iw : bw - *ix;
    *ih = (int)(sh/(*zy)+1);
    *ih = *ih < bh - *iy ? *ih : bh - *iy;
    if (*px < 0) *px = 0; 
    if (*py < 0) *py = 0; 
}

template <class T>
void 
ImageWindow<T>::
DrawBuffer(void)
{
    if (!buffer || !buffer->GetBuffer()) return;
    int w, h, d;
    int ix, iy, iw, ih;
    double px, py, zx, zy;
    GetDrawParameters(&ix, &iy, &iw, &ih, &px, &py, &zx, &zy);
    T *p = buffer->GetBuffer(&w, &h, &d);
    glPixelStorei(GL_UNPACK_ROW_LENGTH, w);
    glPixelStorei(GL_UNPACK_SKIP_ROWS, iy);
    glPixelStorei(GL_UNPACK_SKIP_PIXELS, ix);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glRasterPos2f(px, py);
    glPixelZoom(zx, zy);
    switch (d) {
        case 1:
            GlutWMDrawPixels(iw, ih, GL_LUMINANCE, p);
            break;
        case 3:
            GlutWMDrawPixels(iw, ih, GL_RGB, p);
            break;
        case 4:
            GlutWMDrawPixels(iw, ih, GL_RGBA, p);
            break;
        default:
            errorf(("Invalid buffer dimension %d", d));
            break;
    }
}

#endif // IMAGEWINDOW_H
