#ifndef MATRIX_H
#define MATRIX_H

#include <cmath>
#include <cstdio>
#include <cstring>

#include "scalar.h"
#include "vector.h"

template <int M, int N, class V = double> class Matrix {
public:
    Matrix(void) 
    { 
        for (int i = 0; i < M*N; i++) 
            m[i] = 0;
    }

    Matrix(const V *n, int skip = 1) 
    { 
        for (int i = 0; i < M*N; i++) 
            m[i] = n[i*skip];
    }

    Vector<N> Row(int i) const 
    { 
        i *= 3; 
        return Vector<N>(m+i); 
    }

    Vector<M> Column(int j) const 
    { 
        return Vector<M>(m+j, N); 
    }

    const V &operator[](int c) const 
    {
       return m[c]; 
    }

    V &operator[](int c)
    {
       return m[c]; 
    }

    Matrix<M, N, V> operator-() const 
    { 
        Matrix<M, N, V> r(m); 
        for (int i = 0; i < M*N; i++) 
            r[i] = -r[i]; 
        return r; 
    }

    Matrix<M, N, V> operator-(const Matrix<M, N, V>& n) const
    { 
        Matrix<M, N, V> r(m); 
        for (int i = 0; i < M*N; i++) 
            r[i] -= n[i]; 
        return r; 
    }

    Matrix<M, N, V>& operator-=(const Matrix<M, N, V>& n)
    { 
        for (int i = 0; i < M*N; i++) 
            m[i] -= n[i]; 
        return *this; 
    }

    Matrix<M, N, V> operator+(const Matrix<M, N, V>& n) const
    { 
        Matrix<M, N, V> r(m); 
        for (int i = 0; i < M*N; i++) 
            r[i] += n[i]; 
        return r; 
    }

    Matrix<M, N, V>& operator+=(const Matrix<M, N, V>& n)
    { 
        for (int i = 0; i < M*N; i++) 
            m[i] += n[i]; 
        return *this; 
    }

    Matrix<M, N, V> operator/(const V a) const
    { 
        Matrix<M, N, V> r(m);
        V inv_a = 1.0/a;
        for (int i = 0; i < M*N; i++) 
            r[i] *= inv_a;
        return r;
    }

    Matrix<M, N, V> operator*(const V a) const
    { 
        Matrix<M, N, V> r(m);
        for (int i = 0; i < M*N; i++) 
            r[i] *= a;
        return r;
    }

    Matrix<M, N, V>& operator/=(const V a)
    { 
        V inv_a = 1.0/a;
        for (int i = 0; i < M*N; i++) 
            m[i] *= inv_a;
        return *this; 
    }

    Vector<M,V> operator*(const Vector<M,V> &v) const
    { 
        Vector<M,V> r;
        for (int i = 0; i < M; i++) {
            for (int j = 0; j < N; j++) {
                r[i] += v[j]*m[i*N+j];
            }
        }
        return r;
    }

    bool operator==(const Matrix<M, N, V>& n) const
    { 
        for (int i = 0; i < N*M; i++)
            if (!Equal(m[i], n[i])) return false;
        return true;
    }
    bool operator!=(const Matrix<M, N, V>& n) const
    { return !(*this == n); }


private:
    V m[M*N];
};

typedef Matrix<2, 2> Matrix2;
typedef Matrix<3, 3> Matrix3;
typedef Matrix<4, 4> Matrix4;

//=======================================================================
// Auxiliar functions
//=======================================================================
template <int M, int N, class V>
inline Matrix<M, N, V> operator*(const V& a, const Matrix<M, N, V> &u)
{
    return u*a;
}

template <int M, int N, int P, class V>
inline Matrix<M, P, V> operator*(const Matrix<M, N, V>& m, 
        const Matrix<N, P, V> &n)
{
    Matrix<M, P> r;
    for (int i = 0; i < M; i++) {
        for (int j = 0; j < P; j++) {
            for (int k = 0; k < N; k++) {
                r[i*P+j] += m[i*N+k]*n[k*P+j];
            }
        }
    }
    return r;
}

template <int N, class V>
inline Matrix<N, N, V> Identity(void)
{
    Matrix<N,N> r;
    for (int i = 0; i < N*N; i += N+1) 
        r[i] = 1;
    return r;
}

template <int N, class V>
inline Matrix<N, N, V> Outer(const Vector<N, V> &a, const Vector<N, V> &b)
{
    Matrix<N, N, V> r;
    int i, j;
    for (i = 0; i < N; i++)
        for (j = i; j < N; j++)
            r[i*N+j] = a[i]*b[j];
    for (i = 0; i < N; i++)
        for (j = 0; j < i; j++)
            r[i*N+j] = r[j*N+i];
    return r;
}

template <class V>
inline Matrix<3, 3, V> Cross(const Vector<3, V> &t)
{
    Matrix3 r;
    r[0] = 0;      r[1] = -t[2]; r[2] = t[1];
    r[3] = t[2];   r[4] = 0;     r[5] = -t[0];
    r[6] = -t[1];  r[7] = t[0];  r[8] = 0;
    return r;
}

template <class V>
inline Matrix<3, 3, V> Rodrigues(const Vector<3, V> &w)
{
    V a = w.Norm();
    Matrix3 I = Identity<3,V>();
    if (Zero(a)) return I;
    Vector3 n = w/a;
    V c = cos(a);
    V s = sin(a);
    return c*I + (I - c*I)*Outer(n, n) + s*Cross(n);
}

#endif // MATRIX_H

