#include <cstdio>

#include "error.h"
#include "pngimage.h" 
#include "bufferops.h"

static void Uchar2Ushort(BufferUshort &data)
{
    int width, height, dimension;
    Ushort *ds = data.GetBuffer(&width, &height, &dimension);
    Uchar *dc = (Uchar *) ds;
    for (int i = 0; i < height; i++) {
        for (int j = width*dimension-1; j >= 0; j--) {
            ds[i*width*dimension+j] = (Ushort)
                (dc[i*width*dimension*2 + j] << 8);
        }
    }
}

static void Ushort2Uchar(BufferUshort &data)
{
    int width, height, dimension;
    Ushort *ds = data.GetBuffer(&width, &height, &dimension);
    Uchar *dc = (Uchar *) ds;
    for (int i = 0; i < height; i++) {
        for (int j = 0; j < width*dimension; j++) {
            dc[i*width*dimension*2 + j] = (Uchar) 
                (ds[i*width*dimension+j] >> 8);
        }
    }
}

PngImage::PngImage()
{
    long int a = 1;
    swap = (*((unsigned char *) &a) == 1);
    rows = NULL;
    nrows = 0;
}

PngImage::~PngImage()
{
    if (rows) free(rows);
}

int
PngImage::
AllocateReadStructures(void)
{
    read_struct = NULL; info_struct = NULL; end_info_struct = NULL;
    read_struct = png_create_read_struct(
            PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
    if (!read_struct) {
        errorf(("Out of memory"));
        return 0;
    }
    info_struct = png_create_info_struct(read_struct);
    if (!info_struct) {
        errorf(("Out of memory"));
        goto destroy;
    }
    end_info_struct = png_create_info_struct(read_struct);
    if (!end_info_struct) {
        errorf(("Out of memory"));
        goto destroy;
    }
    return 1;
destroy:
    png_destroy_read_struct(&read_struct, &info_struct, &end_info_struct);
    return 0;
}

int 
PngImage::
Store(BufferUchar &other)
{
    return Translate<Uchar, Ushort>(other, data);
}

int 
PngImage::
Load(const BufferUchar &other)
{
    bit_depth = 8;
    return Translate<Ushort, Uchar>(data, other);
}

int 
PngImage::
SetupRead(void)
{
    int width, height, dimension;
    int color_type;
    // find out image properties
    width = png_get_image_width(read_struct, info_struct);
    height = png_get_image_height(read_struct, info_struct);
    color_type = png_get_color_type(read_struct, info_struct);
    bit_depth = png_get_bit_depth(read_struct, info_struct);
    if (color_type & PNG_COLOR_MASK_ALPHA) {
        png_set_expand(read_struct);
        dimension = 1;
    } else 
        dimension = 0;
    if (color_type & PNG_COLOR_MASK_COLOR) {
        dimension += 3;
        if (color_type & PNG_COLOR_MASK_PALETTE) png_set_expand(read_struct);
    } else dimension += 1;
    if (bit_depth < 8) {
        png_set_expand(read_struct);
        bit_depth = 8;
    }
    if (swap) 
		png_set_swap(read_struct);
	png_read_update_info(read_struct, info_struct);
	// allocate buffers accordingly
    if (!data.Resize(width, height, dimension)) {
        errorf(("Out of memory"));
        return 0;
    }
    if (!SetupRows()) {
        errorf(("Unable to allocate row pointers"));
        return 0;
    }
    return 1;
}
 
int 
PngImage::
Load(FILE *fp)
{
    int ret = 0;
    Uchar signature[8];
    if (fread(signature, 1, 8, fp) < 8) {
        errorf(("Unable to read from file"));
        goto done;
    }
    if (png_sig_cmp(signature, 0, 8)) {
        errorf(("File is in PNG format"));
        goto done;
    }
    if (!AllocateReadStructures()) {
        errorf(("Unable to allocate read structures"));
        goto done;
    }
    if (setjmp(read_struct->jmpbuf)) {
        errorf(("Loading failed"));
        goto done;
    }
    png_init_io(read_struct, fp);
    png_set_sig_bytes(read_struct, 8);
    png_read_info(read_struct, info_struct);
    if (!SetupRead()) {
        errorf(("Unable to setup read operation"));
        goto done;
    }
    png_read_image(read_struct, rows);
    if (bit_depth == 8) Uchar2Ushort(data);
    ret = 1;
done:
    png_destroy_read_struct(&read_struct, &info_struct, &end_info_struct);
    return ret;
}

int 
PngImage::
Load(const char *name)
{
    FILE *fp = fopen(name, "rb");
    if (!fp) {
        errorf(("Unable to open file '%s' for reading", name));
        return 0;
    }
    if (!Load(fp)) {
        fclose(fp);
        return 0;
    } else {
        fclose(fp);
        return 1;
    }
}

int
PngImage::
AllocateWriteStructures(void)
{
    write_struct = NULL; info_struct = NULL;
    write_struct = png_create_write_struct(
            PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
    if (!write_struct) {
        errorf(("Out of memory"));
        return 0;
    }
    info_struct = png_create_info_struct(write_struct);
    if (!info_struct) {
        errorf(("Out of memory"));
        png_destroy_write_struct(&write_struct, NULL);
        return 0;
    }
    return 1;
}

typedef struct t_mem_io_ {
    const unsigned char *data;
    size_t position, size;
} t_mem_io;
typedef t_mem_io *p_mem_io;

static void read_mem_io(png_structp read_struct, png_bytep data, 
        png_size_t length)
{
    p_mem_io mem = (p_mem_io) png_get_io_ptr(read_struct);
    if (mem->position + length > mem->size)
        length = mem->size - mem->position;
    memcpy(data, mem->data+mem->position, length);
    mem->position += length;
}

int PngImage::Load(const unsigned char *chunk, size_t size)
{
    t_mem_io mem = {chunk, 0, size};
    int ret = 0;
    if (png_sig_cmp((png_bytep) chunk, 0, 8)) {
        errorf(("Blob is not in PNG format"));
        return ret;
    }
    if (!AllocateReadStructures()) {
        errorf(("Unable to create read-structures"));
        return ret;
    }
    if (setjmp(read_struct->jmpbuf)) {
        errorf(("Error loading blob"));
        goto done;
    }
    png_set_read_fn(read_struct, &mem, read_mem_io);
    png_read_info(read_struct, info_struct);
    if (!SetupRead()) {
        errorf(("Unable to setup read operation"));
        goto done;
    }
    png_read_image(read_struct, rows);
    if (bit_depth == 8) Uchar2Ushort(data);
    ret = 1;
done:
    png_destroy_read_struct(&read_struct, &info_struct, &end_info_struct);
    return ret;
}


int 
PngImage::
Store(FILE *fp)
{
    int ret = 0;
    if (!AllocateWriteStructures()) {
        errorf(("Unable to allocate write structures"));
        goto done;
    }
    if (setjmp(write_struct->jmpbuf)) {
        errorf(("Storing failed"));
        goto done;
    }
    png_init_io(write_struct, fp);
    if (!SetupWrite()) {
        errorf(("Unable to setup write operation"));
        goto done;
    }
	if (bit_depth == 8) Ushort2Uchar(data);
	png_write_image(write_struct, rows);
    png_write_end(write_struct, info_struct);
	if (bit_depth == 8) Uchar2Ushort(data);
    ret = 1;
done:
    png_destroy_write_struct(&write_struct, &info_struct);
    return ret;
}

int
PngImage::
SetupWrite(void)
{
    int width = data.GetWidth();
    int height = data.GetHeight();
    int dimension = data.GetDimension();
    int color_type;
    switch (dimension) {
        case 1:
            color_type = PNG_COLOR_TYPE_GRAY;
            break;
        case 2:
            color_type = PNG_COLOR_TYPE_GRAY_ALPHA;
            break;
        case 3:
            color_type = PNG_COLOR_TYPE_RGB;
            break;
        case 4:
            color_type = PNG_COLOR_TYPE_RGB_ALPHA;
            break;
        default:
            errorf(("Unable to write image with %d dimensions", dimension));
            return 0;
    }
    png_set_filter(write_struct, 0, PNG_FILTER_NONE);
    png_set_IHDR(write_struct, info_struct, width, height, bit_depth, 
            color_type, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT,
            PNG_FILTER_TYPE_DEFAULT);
    png_write_info(write_struct, info_struct);
	if (swap)
		png_set_swap(write_struct);
    if (!SetupRows()) {
        errorf(("Unable to allocate row pointers"));
        return 0;
    }
    return 1;
}

int
PngImage::
SetupRows(void)
{
    int width, height, dimension;
    Ushort *dp = data.GetBuffer(&width, &height, &dimension);
    if (nrows < height) {
        if (rows) free(rows);
        rows = (png_bytepp) malloc(height*sizeof(png_bytep *));
        if (!rows) {
            errorf(("Out of memory"));
            return 0; 
        }
        nrows = height;
    }
    rows[height-1] = (png_bytep) dp;
    for (int i = height-2; i >= 0; i--)
        rows[i] = (png_bytep) ((Ushort *) rows[i+1] + width*dimension);
    return 1;
}

int PngImage::Store(const char *name)
{
    FILE *fp = fopen(name, "wb");
    if (!fp) {
        errorf(("Unable to open file '%s' for writing", name));
        return 0;
    }
    if (!Store(fp)) {
        fclose(fp);
        return 0;
    } else {
        fclose(fp);
        return 1;
    }
}

int PngImage::Store(BufferUshort &other)
{
    return data.Store(other);
}

int PngImage::Load(const BufferUshort &other)
{
    bit_depth = 16;
    return data.Load(other);
}


int PngImage::GetWidth()
{
    return data.GetWidth();
}

int PngImage::GetHeight()
{
    return data.GetHeight();
}
