//=======================================================================
// Quaternion class implementation.
// 19/7/99, Diego Nehab
//=======================================================================

#ifndef QUATERNION_H
#define QUATERNION_H

#include "scalar.h"
#include "vector.h"

class Quaternion {
public:
    Quaternion (double x = 0.0, double y = 0.0, double z = 0.0, double w = 0.0) 
    { 
        q[0] = x; q[1] = y; q[2] = z; q[3] = w; 
    }

    Quaternion (double a, Vector3 v)
    { 
        a *= SCALAR_TORADIAN;
        double n = v.Norm(); 
        if (Zero(n)) n = 1;
        double inv_n = 1.0/n;
        double s = sin(a/2);
        q[0] = v[0]*s*inv_n; q[1] = v[1]*s*inv_n; 
        q[2] = v[2]*s*inv_n; q[3] = cos(a/2);
    }

    Quaternion operator-() const 
    { 
        return Quaternion(-q[0], -q[1], -q[2], -q[3]); 
    }

    Quaternion operator-(const Quaternion& r) const
    { 
        return Quaternion(q[0]-r[0], q[1]-r[1], q[2]-r[2], q[3]-r[3]); 
    }

    Quaternion& operator-=(const Quaternion& r)
    { 
        q[0] -= r[0]; q[1] -= r[1]; q[2] -= r[2]; q[3] -= r[3]; 
        return *this; 
    }

    Quaternion operator+(const Quaternion& r) const
    { 
        return Quaternion(q[0]+r[0], q[1]+r[1], q[2]+r[2], q[3]+r[3]); 
    }

    Quaternion& operator+=(const Quaternion& r)
    { 
        q[0] += r[0]; q[1] += r[1]; q[2] += r[2]; q[3] += r[3]; 
        return *this; 
    }

    Quaternion operator*(const Quaternion& r) const
    { 
        return Quaternion(
            q[3]*r[0] + q[0]*r[3] + q[1]*r[2] - q[2]*r[1],
            q[3]*r[1] + q[1]*r[3] + q[2]*r[0] - q[0]*r[2],
            q[3]*r[2] + q[2]*r[3] + q[0]*r[1] - q[1]*r[0],
            q[3]*r[3] - q[0]*r[0] - q[1]*r[1] - q[2]*r[2]); 
    }

    Quaternion operator*(const double a)
    {
        return Quaternion(a*q[0], a*q[1], a*q[2], a*q[3]);
    }


    Quaternion& operator*=(const Quaternion& r)
    { 
        double x = q[3]*r[0] + q[0]*r[3] + q[1]*r[2] - q[2]*r[1];
        double y = q[3]*r[1] + q[1]*r[3] + q[2]*r[0] - q[0]*r[2];
        double z = q[3]*r[2] + q[2]*r[3] + q[0]*r[1] - q[1]*r[0];
        double w = q[3]*r[3] - q[0]*r[0] - q[1]*r[1] - q[2]*r[2]; 
        q[0] = x; q[1] = y; q[2] = z; q[3] = w; 
        return *this; 
    }

    Quaternion operator/(const double a) const
    { 
        double inv_a = 1.0/a;
        return Quaternion(q[0]*inv_a, q[1]*inv_a, q[2]*inv_a, q[3]*inv_a); 
    }

    Quaternion& operator/=(const double a)
    { 
        double inv_a = 1.0/a;
        q[0] *= inv_a; q[1] *= inv_a; q[2] *= inv_a; 
        q[3] *= inv_a; 
        return *this; 
    }

    bool operator==(const Quaternion& r) const
    { 
        return Equal(q[0], r[0]) && Equal(q[1], r[1]) && 
            Equal(q[2], r[2]) && Equal(q[3], q[3]); 
    }

    bool operator!=(const Quaternion& r)const
    { 
        return !Equal(q[0],r[0]) || !Equal(q[1],r[1]) || 
            !Equal(q[2],r[2]) || !Equal(q[3], q[3]); 
    }

    double Rotation(Vector3 *v) const
    { 
        Vector3 &u = *v;
        Quaternion n = *this; 
        n /= n.Norm();
        u[X] = n[0]; u[Y] = n[1]; u[Z] = n[2]; 
        return SCALAR_TODEGREE*2*acos(n[3]);
    }

    double Rotation(double *x, double *y, double *z) const
    { 
        Quaternion n = *this; 
        n /= n.Norm();
        *x = n[0]; *y = n[1]; *z = n[2]; 
        return SCALAR_TODEGREE*2*acos(n[3]);
    }

    const double &operator[](int c) const
    { 
        return q[c]; 
    }

    double &operator[](int c)
    { 
        return q[c]; 
    }

    double Norm(void) const
    { 
        return sqrt(q[0]*q[0] + q[1]*q[1] + q[2]*q[2] + q[3]*q[3]); 
    }

private:
    // quaternion coordinates
    double q[4];
};

//=======================================================================
// Auxiliar functions
//=======================================================================
inline Quaternion operator*(const double &a, const Quaternion &q)
{
    return q*a;
}

#endif // QUATERNION_H

