#include <cstdio>
#include <cstdlib>
#include <cstring>

#include "error.h"
#include "tar.h"

static size_t checksum(p_block block)
{
    // checksum is sum of all bytes in block
    size_t i, sum = 0;
    for (i = 0; i < sizeof(u_block); i++) 
        sum += block->data[i];
    // checksum field is considered to be blank for computation purposes
    for (i = 0; i < 8; i++)
        sum -= block->header.checksum[i];
    return sum + 8*' '; 
}

static bool is_eof(p_block block)
{
    unsigned char *raw = (unsigned char *) block;
    for (size_t i = 0; i < sizeof(u_block); i++)
        if (raw[i]) return false;
    return true;
}

static void header(p_block block, const char *name, size_t size)
{
    memset(block, 0, sizeof(u_block));
    strncpy(block->header.name, name, sizeof(block->header.name)-1); 
    sprintf(block->header.size, "%011lo ", (unsigned long) size); 
    block->header.linkflag = TAR_NORMAL;
    sprintf(block->header.mode, "%06lo", 
            TAR_UREAD | TAR_UWRITE | TAR_GREAD | TAR_OREAD);
    sprintf(block->header.checksum, "%06lo ", (unsigned long) checksum(block)); 
}

Tar::
Tar(void)
{
    fp = NULL;
    append_pos = 0;
    mode = TAR_CLOSED;
}

Tar::
~Tar()
{
    Close();
}


int
Tar::
Load(const char *name)
{
    Close();
    index.Empty();
    fp = fopen(name, "rb");
    if (!fp) {
        errorf(("Unable to open file '%s' for reading", name));
        return 0;
    }
    mode = TAR_READ;
    u_block buffer;
    while (1) {
        // get one header worth of data
        int ret = fread(&buffer, 1, sizeof(u_block), fp);
        if (ret <= 0) {
            warnf(("Unexpected end of file")); 
            break; 
        }
        // did we get to the end of the archive?
        if (is_eof(&buffer)) 
            break; 
        // if not a valid header, skip it
        if ((long) checksum(&buffer) 
                != strtol(buffer.header.checksum, NULL, 8)) {
            warnf(("Skipping invalid header"));
            continue;
        }
        // if this entry is a regular file, index it
        size_t size = strtol(buffer.header.size, NULL, 8);
        if (buffer.header.linkflag == TAR_NORMAL ||
            buffer.header.linkflag == TAR_OLD_NORMAL) {
            t_entry entry = {ftell(fp), size};
            if (!index.PushBack(entry)) 
                warnf(("Out of memory"));
        }
        // jump to next entry
        size = (size + 511)/512;
        if (fseek(fp, size*512, SEEK_CUR) < 0) {
            warnf(("Unable to seek to next entry")); 
            break;
        }
    }
    return 1;
}

int 
Tar::
Read(int entry, unsigned char *data)
{
    if (!fp || mode != TAR_READ) {
        errorf(("Archive has not been openned for reading")); 
        return 0;
    }
    if (entry < 0 || entry >= index.GetSize()) {
        errorf(("Entry %d is out of range", entry));
        return 0;
    }
    if (fseek(fp, index[entry].start, SEEK_SET) < 0) {
        errorf(("Unable to seek to start of entry %d", entry));
        return 0;
    }
    size_t size = index[entry].size;
    if (fread(data, 1, size, fp) < size) {
        errorf(("Unexpected end of archive at entry %d", entry));
        return 0;
    }
    return 1;
}


int
Tar::
Create(const char *name)
{
    Close();
    fp = fopen(name, "wb");
    if (!fp) {
        errorf(("Unable to open file '%s' for writing", name));
        return 0;
    }
    index.Empty();
    mode = TAR_WRITE;
    append_pos = 0;
    return 1;
}

FILE *
Tar::
AppendBegin(const char *name)
{
    if (!fp || mode != TAR_WRITE) {
        errorf(("Archive has not been openned for writing"));
        return NULL;
    }
    if (fseek(fp, append_pos+512, SEEK_SET) < 0) {
        errorf(("Unable to seek to entry start"));
        return NULL;
    }
    strncpy(append_name, name, sizeof(append_name)-1); 
    return fp;
}

int 
Tar::
AppendEnd(void)
{
    char zero[512];
    memset(zero, 0, sizeof(zero));
    if (!fp || mode != TAR_WRITE) {
        errorf(("Archive has not been openned for writing"));
        return 0;
    }
    // pad to 512 byte boundary
    size_t size = ftell(fp) - append_pos - 512;
    size_t skip = size % 512;
    skip = skip > 0 ? 512 - skip : 0;
    if (fwrite(zero, 1, skip, fp) < skip) {
        errorf(("Unable to pad entry"));
        return 0;
    }
    size_t next_append_pos = ftell(fp);
    // goto header position in file
    if (fseek(fp, append_pos, SEEK_SET) < 0) {
        errorf(("Unable to seek to entry header"));
        return 0;
    }
    // build header
    u_block block;
    header(&block, append_name, size);
    // store header
    if (fwrite(&block, 1, sizeof(block), fp) < sizeof(block)) {
        errorf(("Unable to write entry header"));
        return 0;
    }
    append_pos = next_append_pos;
    return 1;
}

int
Tar::
Close(void)
{
    if (fp) {
        // if we are in write mode, we have to write the end-of-file block
        if (mode == TAR_WRITE) {
            if (fseek(fp, 0, SEEK_END) < 0) {
                errorf(("Unable to seek to the end of file"));
                return 0;
            }
            char zero[512];
            memset(zero, 0, sizeof(zero));
            if (fwrite(zero, 1, sizeof(zero), fp) < sizeof(zero)) {
                errorf(("Unable write end-of-file block"));
                return 0;
            }
        }
        fclose(fp);
    }
    fp = NULL;
    return 1;
}


size_t
Tar::
GetNEntries(void) const
{
    return index.GetSize();
}

size_t
Tar::
GetSize(int entry) const
{
    if (entry < 0 || entry >= (int) GetNEntries()) {
        errorf(("Entry is out of range"));
        return 0;
    }
    return index[entry].size;
}
