#ifndef VECTOR_H
#define VECTOR_H

#include <cmath>
#include <cstdio>

#include "error.h"
#include "scalar.h"

enum {X, Y, Z, W};

template <int N, class V = double> class Vector {
public:
    // constructors
    Vector(void) 
    { 
        for (int i = 0; i < N; i++) 
            v[i] = 0; 
    }
    Vector(V x, V y) 
    { 
        assertf(N == 2, ("Wrong constructor for vector!"));
        v[0] = x; v[1] = y;
    }
    Vector(V x, V y, V z) 
    { 
        assertf(N == 3, ("Wrong constructor for vector!"));
        v[0] = x; v[1] = y; v[2] = z;
    }
    Vector(V x, V y, V z, V w) 
    { 
        assertf(N == 4, ("Wrong constructor for vector!"));
        v[0] = x; v[1] = y; v[2] = z; v[3] = w;
    }

    // for homogeneous coordinates
    Vector(const Vector<N+1, V> &u) 
    {   V inv_n;
        if (!Zero(u[N])) inv_n = 1/u[N];
        else inv_n = 1;
        for (int i = 0; i < N; i++) 
            v[i] = u[i]*inv_n; 
    }
    operator Vector<N+1, V> ()
    { 
        Vector<N+1, V> u;
        for (int i = 0; i < N; i++)
            u[i] = v[i];
        u[N] = 1;
        return u; 
    }

    // arithmetic
    Vector<N, V> operator-() const 
    { 
        Vector<N, V> t;
        for (int i = 0; i < N; i++) t[i] = -v[i];
        return t;
    }
    Vector<N, V> operator-(const Vector<N, V>& u) const
    { 
        Vector<N, V> t;
        for (int i = 0; i < N; i++) t[i] = v[i]-u[i];
        return t;
    };
    Vector<N, V> operator+(const Vector<N, V>& u) const
    { 
        Vector<N, V> t;
        for (int i = 0; i < N; i++) t[i] = v[i]+u[i];
        return t;
    };
    Vector<N, V> operator/(const V &a) const
    { 
        V inv_a = 1.0/a;
        Vector<N, V> t;
        for (int i = 0; i < N; i++) t[i] = v[i]*inv_a;
        return t;
    }
    Vector<N, V> operator*(const V &a) const
    { 
        Vector<N, V> t;
        for (int i = 0; i < N; i++) t[i] = v[i]*a;
        return t;
    }
    Vector<N, V>& operator-=(const Vector<N, V>& u)
    { 
        for (int i = 0; i < N; i++) v[i] -= u[i]; 
        return *this;
    }
    Vector<N, V>& operator+=(const Vector<N, V>& u)
    { 
        for (int i = 0; i < N; i++) v[i] += u[i]; 
        return *this;
    }
    Vector<N, V>& operator/=(const V &a)
    { 
        V inv_a = 1.0/a;
        for (int i = 0; i < N; i++) v[i] *= inv_a;
        return *this;
    }
    Vector<N, V>& operator*=(const V &a)
    { 
        for (int i = 0; i < N; i++) v[i] *= a;
        return *this;
    }

    // comparisons
    bool operator==(const Vector<N, V>& u) const
    { 
        for (int i = 0; i < N; i++)
            if (!Equal(v[i], u[i])) return false;
        return true;
    }
    bool operator!=(const Vector<N, V>& u)const
    { return !(*this == u); }

    // indexing
    V &operator[](int c)
    { return v[c]; };
    const V& operator[](int c) const
    { return v[c]; };

    // vector operations
    V Dot(const Vector<N, V> &u) const
    {
        V dot = 0.0;
        for (int i = 0; i < N; i++) dot += v[i]*u[i];
        return dot;
    }

    V Norm(void) const
    { return sqrt(Dot(*this)); } 

private:
    V v[N];
};

typedef Vector<2> Vector2;
typedef Vector<3> Vector3;
typedef Vector<4> Vector4;

template <int N, class V> Vector<N, V> operator*(V a, const Vector<N, V> &v)
{
    return v*a;
}

template <int N, class V> V Norm(const Vector<N, V> &v)
{
    return v.Norm();
}

template <int N, class V> V Dot(const Vector<N, V> &u, const Vector<N, V> &v)
{
    return u.Dot(v);
}

template <class V>
inline V Phase(const Vector<2, V> &v)
{
    return atan2(v[Y], v[X]); 
}

template <class V>
inline Vector<3, V> Cross(const Vector<3, V> &u, const Vector<3, V> &v)
{
    return 
        Vector<3, V>(u[1]*v[2]-u[2]*v[1], 
                u[2]*v[0]-u[0]*v[2], 
                u[0]*v[1]-u[1]*v[0]);
}

template <int N, class V>
inline bool Zero(const Vector<N, V> &v)
{
    for (int i = 0; i < N; i++) {
        if (!Zero(v[i])) return false;
    }
    return true;
}

#endif // VECTOR_H
